#!/bin/sh

PHP=`which php`

if [ "${PHP}" = "" ] || [ ! -e "${PHP}" ] ; then
	echo -n " Missing PHP binary!" ;
	exit 1 ;
fi

mkdir -p /opt/ispprotect_bandaemon
cp -r conf.d tpl ispp_bandaemon v* /opt/ispprotect_bandaemon/
if [ -d "/usr/share/php5" ] ; then
	cp ispp_bd_postlog.php /usr/share/php5/
	chmod 755 /usr/share/php5/ispp_bd_postlog.php
fi
if [ -d "/usr/share/php7.0" ] ; then
	cp ispp_bd_postlog.php /usr/share/php7.0/
	chmod 755 /usr/share/php7.0/ispp_bd_postlog.php
fi
if [ -d "/usr/share/php" ] ; then
	cp ispp_bd_postlog.php /usr/share/php/
	chmod 755 /usr/share/php/ispp_bd_postlog.php
fi
if [ -e "/opt/ispp_bd_postlog.php" ] ; then
	rm -f /opt/ispp_bd_postlog.php ;
fi

if [ -e "license.key" ] ; then
	cp license.key /opt/ispprotect_bandaemon/ ;
fi
chown root:root /opt/ispprotect_bandaemon
chmod -R 640 /opt/ispprotect_bandaemon
chmod -R 750 /opt/ispprotect_bandaemon/ispp_bandaemon

if [ -e "/etc/debian_version" ] ; then
    cp etc/init.d/ispprotect_bandaemon /etc/init.d/
else
    cp etc/init.d/ispprotect_bandaemon_centos /etc/init.d/ispprotect_bandaemon
fi
chmod 755 /etc/init.d/ispprotect_bandaemon
update-rc.d ispprotect_bandaemon defaults

if [ -e "/bin/systemctl" ] ; then
	/bin/systemctl daemon-reload
	/bin/systemctl enable ispprotect_bandaemon
	/bin/systemctl daemon-reload
fi

if [ -e "/opt/ispconfig3_bandaemon" ] ; then
	echo "========"
	echo "WARNING!"
	echo "========"
	echo "Paths and file names have changed. Trying to update neccessary files automatically."
	echo "Please delete the /opt/ispconfig3_bandaemon directory once everything is working correctly."
	echo "========"
	echo ""
	if [ -e "/bin/systemctl" ] ; then
		service ispconfig3_bandaemon stop
	else
		/etc/init.d/ispconfig3_bandaemon stop ;
	fi
	cp -a /opt/ispconfig3_bandaemon/config.inc.php /opt/ispprotect_bandaemon/ ;
	cp -a /opt/ispconfig3_bandaemon/conf.d /opt/ispprotect_bandaemon/ ;
	cp -a /opt/ispconfig3_bandaemon/license.key /opt/ispprotect_bandaemon/ ;
	if [ -e "/etc/cron.d/ispconfig3_bandaemon" ] ; then
		rm -f /etc/cron.d/ispconfig3_bandaemon ;
		echo "15 0 * * * root /opt/ispprotect_bandaemon/ispp_bandaemon --report > /dev/null 2>&1" > /etc/cron.d/ispprotect_bandaemon ;
	fi
	rm -f /etc/logrotate.d/ispconfig3_bandaemon ;
	echo "/var/log/ispprotect_bandaemon.log {
	daily
	rotate 7
	missingok
	create 640 root root
	compress
	notifempty
}" > /etc/logrotate.d/ispprotect_bandaemon ;

	if [ -e "/etc/init.d/ispconfig3_bandaemon" ] ; then
		if [ -e "/bin/systemctl" ] ; then
			/bin/systemctl daemon-reload ;
			/bin/systemctl disable ispconfig3_bandaemon ;
		fi
		rm -f /etc/init.d/ispconfig3_bandaemon ;
		if [ -e "/bin/systemctl" ] ; then
			/bin/systemctl daemon-reload ;
		fi
	fi
fi

PWORD=$(date +%N%s | sha256sum | head -c 32 ; echo) ;

HASH=$(hostname -f) ;
# Check if hostname contains a dot using case statement (POSIX compatible)
case "$HASH" in
	*.*) 
		echo "Server hostname: $HASH" ;
		;;
	*)
		HASH=$(hostname) ;
		case "$HASH" in
			*.*)
				echo "Server hostname: $HASH" ;
				;;
			*)
				HASH=$(date +%N%s | sha256sum | head -c 32 ; echo) ;
				echo "Using $HASH as server ident, as hostname is too short and might not be unique." ;
				;;
		esac
		;;
esac


if [ ! -e "/opt/ispprotect_bandaemon/config.inc.php" ] ; then
	read -p "Create database on this server (if no, you have to create it by yourself) [Y/n]: " CREAT
	if [ "$CREAT" = "" ] || [ "$CREAT" = "Y" ] || [ "$CREAT" = "y" ] ; then
		mysql --defaults-file=/etc/mysql/debian.cnf -e "CREATE DATABASE IF NOT EXISTS ispprotect_bandaemon"
		mysql --defaults-file=/etc/mysql/debian.cnf -e "GRANT ALL PRIVILEGES ON ispprotect_bandaemon.* TO 'ispp_bandaemon'@'localhost' IDENTIFIED BY '${PWORD}'";
	else
		PWORD="";
		echo "NOT creating database. Edit config.inc.php according to your needs.";
	fi
fi

WWWPATH="/var/www"
if [ ! -d "$WWWPATH" ] ; then
	if [ -d "/srv/www" ] ; then
		WWWPATH="/srv/www";
	else
		echo ""
		echo "WARNING!! no /var/www or /srv/www found! Disabling all WWW bans. Set BANDAEMON_WWW_ROOT.";
		echo ""
		WWWPATH=""
	fi
fi

echo "<?php
/** config section **/

DEFINE('DB_HOST', 'localhost');
DEFINE('DB_NAME', 'ispprotect_bandaemon');
DEFINE('DB_USER', 'ispp_bandaemon');
DEFINE('DB_PASS', '${PWORD}');

// use INPUT or PREROUTING chain in iptables
// if you have problems with prerouting (e. g. on vserver) set to INPUT
define('BANDAEMON_TABLE', 'PREROUTING');

// append iptable chain rules (APPEND) or prepend (INSERT)
// set to APPEND if you need your own rules to be processed first
define('BANDAEMON_MODE', 'INSERT');

// use syslog for logging (/var/log/ispprotect_bandaemon.log if false)
define('BANDAEMON_SYSLOG', false);

// if true log info msg, too
define('BANDAEMON_VERBOSE', false);

// if true, the file where a hit was found is written to log in VERBOSE mode
define('BANDAEMON_LOGNAME', false);

// path to your website base path or boolean false if none (NOT '')
define('BANDAEMON_WWW_ROOT', '${WWWPATH}');

// comma-separated list of watch entries to disable
// e.g. 'apachenginx', 'postfix', 'wordpress', 'ssh', 'joomla' or 'bind'
define('BANDAEMON_DISABLE_WATCH', '');

// comma-separated list of watch entries to enable
// this will disable all watches not mentioned here
// BANDAEMON_DISABLE_WATCH will be ignored
// e.g. 'apachenginx', 'postfix', 'wordpress', 'ssh', 'joomla' or 'bind'
define('BANDAEMON_ENABLE_WATCH', '');

// if true do not really ban ips (skip iptable calls)
define('BANDAEMON_TESTMODE', false);

// if set to true, the daemon tries to get the abuse email address for an
// ip that was banned, so you can send a complaint to them
define('BANDAEMON_ABUSE_ADDRESS', false);

// if set to true, the daemon fetches all log entries for an ip that was
// banned and saves it to a file in ./evidence/ together with the abuse ip
// (if above setting is true, too)
define('BANDAEMON_EVIDENCE', false);

// What minimum score should is neccessary to create the evidence file?
// Bans begin at a score of 50.
define('BANDAEMON_EVIDENCE_SCORE', 100);

// if set to an email address, the daily report is sent there
// If you use a centralized database, only activate this on ONE server
// otherwise you get the same report multiple times
define('BANDAEMON_DAILY_REPORT', false);

// you can choose your own, but if multiple severs share a db, make it unique!
define('SERVER_IDENT', '${HASH}');

// shall the ban daemon log flood attempts? If false those are only blocked but not logged.
define('LOG_FLOOD_ATTEMPTS', true);

// comma-separated list (local ips are added automatically)
define('WHITELISTED_IPS', '');

// comma-separated list with score pairs CountryCode:Score
// the score is added to the ip score on logfile hits
// uncomment entry and alter values/add new to change country scores
// define('BANDAEMON_COUNTRY_SCORES', 'VN:20,CN:20,HU:10');

/** for abuse reporting tool **/

// if you want to send abuse reports to abuse teams responsible for offending ips
// please set your sender email address here
define('EMAIL_SENDER', '');

// if you want to send abuse reports to abuse teams responsible for offending ips
// please set your sender name address here
define('EMAIL_SENDER_NAME', 'Abuse Team');

/** end of config section **/
" > /opt/ispprotect_bandaemon/config.inc.php.sample

if [ ! -e "/opt/ispprotect_bandaemon/config.inc.php" ] ; then
	cp /opt/ispprotect_bandaemon/config.inc.php.sample /opt/ispprotect_bandaemon/config.inc.php ;

	echo "/var/log/ispprotect_bandaemon.log {
	daily
	rotate 7
	missingok
	create 640 root root
	compress
	notifempty
}" > /etc/logrotate.d/ispprotect_bandaemon ;

	read -p "Create cron job for daily report via email [y/N]: " CREAT
	if [ "$CREAT" = "Y" ] || [ "$CREAT" = "y" ] ; then

		if [ -e "/etc/cron.d/ispprotect_daemon" ] ; then
			rm -f /etc/cron.d/ispprotect_daemon
		fi
		echo "15 0 * * * root /opt/ispprotect_bandaemon/ispp_bandaemon --report > /dev/null 2>&1" > /etc/cron.d/ispprotect_bandaemon
		echo "Don't forget to set up daily report target email in config.inc.php";
	fi
fi

echo "You should install php5-geoip and geoip-database-contrib it not yet installed."
echo ""
echo "Edit /opt/ispprotect_bandaemon/config.inc.php.sample according to your needs, then rename"
echo "it to config.inc.php (or make a copy). After this you can start the daemon by typing"
echo "'service ispprotect_bandaemon start'"
