#!/bin/sh

#################################
#      ISPProtect BanDaemon     #
#         Uninstaller           #
#################################

# Colors for output
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[1;33m'
BLUE='\033[0;34m'
BOLD='\033[1m'
NC='\033[0m'

# Installation paths
INSTALL_DIR="/opt/ispprotect_bandaemon"
LEGACY_DIR="/opt/ispconfig3_bandaemon"
SERVICE_NAME="ispprotect_bandaemon"
LEGACY_SERVICE="ispconfig3_bandaemon"

# Flags for what to remove
REMOVE_FILES=true
REMOVE_CONFIG=false
REMOVE_DATABASE=false
REMOVE_LOGS=false
REMOVE_EVIDENCE=false

print_banner() {
    echo ""
    echo -e "${RED}${BOLD}╔══════════════════════════════════════╗${NC}"
    echo -e "${RED}${BOLD}║        ISPProtect BanDaemon          ║${NC}"
    echo -e "${RED}${BOLD}║           Uninstaller                ║${NC}"
    echo -e "${RED}${BOLD}╚══════════════════════════════════════╝${NC}"
    echo ""
    echo -e "${YELLOW}This will remove ISPProtect BanDaemon from your system.${NC}"
    echo ""
}

check_root() {
    if [ "$(id -u)" != "0" ]; then
        echo -e "${RED}✗ This script must be run as root${NC}"
        exit 1
    fi
}

detect_installation() {
    echo -e "${BLUE}Detecting installation type...${NC}"
    
    # Check for systemd service
    if [ -f "/etc/systemd/system/$SERVICE_NAME.service" ]; then
        echo -e "${GREEN}✓ Found systemd service${NC}"
        SYSTEMD_INSTALLED=true
    else
        SYSTEMD_INSTALLED=false
    fi
    
    # Check for init.d services
    INITD_INSTALLED=false
    for service in $SERVICE_NAME $LEGACY_SERVICE; do
        if [ -f "/etc/init.d/$service" ]; then
            echo -e "${GREEN}✓ Found init.d service: $service${NC}"
            INITD_INSTALLED=true
        fi
    done
    
    # Check for main installation
    if [ -d "$INSTALL_DIR" ]; then
        echo -e "${GREEN}✓ Found main installation: $INSTALL_DIR${NC}"
        MAIN_INSTALLED=true
    else
        MAIN_INSTALLED=false
    fi
    
    # Check for legacy installation
    if [ -d "$LEGACY_DIR" ]; then
        echo -e "${GREEN}✓ Found legacy installation: $LEGACY_DIR${NC}"
        LEGACY_INSTALLED=true
    else
        LEGACY_INSTALLED=false
    fi
    
    if [ "$SYSTEMD_INSTALLED" = false ] && [ "$INITD_INSTALLED" = false ] && [ "$MAIN_INSTALLED" = false ] && [ "$LEGACY_INSTALLED" = false ]; then
        echo -e "${YELLOW}⚠ No ISPProtect BanDaemon installation found${NC}"
        echo ""
        printf "Continue anyway to clean up any remaining files? [y/N]: "
        read -r CONTINUE
        if [ "$CONTINUE" != "y" ] && [ "$CONTINUE" != "Y" ]; then
            echo "Exiting."
            exit 0
        fi
    fi
    
    echo ""
}

ask_user_preferences() {
    echo -e "${BLUE}Uninstall Options:${NC}"
    echo ""
    
    # Always remove program files
    echo -e "${GREEN}✓ Program files will be removed${NC}"
    
    # Ask about configuration
    printf "Remove configuration files (config.inc.php, license.key)? [y/N]: "
    read -r REMOVE_CONFIG_INPUT
    if [ "$REMOVE_CONFIG_INPUT" = "y" ] || [ "$REMOVE_CONFIG_INPUT" = "Y" ]; then
        REMOVE_CONFIG=true
        echo -e "${YELLOW}✓ Configuration files will be removed${NC}"
    fi
    
    # Ask about database
    printf "Remove database (ispprotect_bandaemon)? [y/N]: "
    read -r REMOVE_DB_INPUT
    if [ "$REMOVE_DB_INPUT" = "y" ] || [ "$REMOVE_DB_INPUT" = "Y" ]; then
        REMOVE_DATABASE=true
        echo -e "${YELLOW}✓ Database will be removed${NC}"
    fi
    
    # Ask about logs
    printf "Remove log files (/var/log/ispprotect_bandaemon.log)? [y/N]: "
    read -r REMOVE_LOGS_INPUT
    if [ "$REMOVE_LOGS_INPUT" = "y" ] || [ "$REMOVE_LOGS_INPUT" = "Y" ]; then
        REMOVE_LOGS=true
        echo -e "${YELLOW}✓ Log files will be removed${NC}"
    fi
    
    # Ask about evidence
    if [ -d "$INSTALL_DIR/evidence" ] || [ -d "$LEGACY_DIR/evidence" ]; then
        printf "Remove evidence files? [y/N]: "
        read -r REMOVE_EVIDENCE_INPUT
        if [ "$REMOVE_EVIDENCE_INPUT" = "y" ] || [ "$REMOVE_EVIDENCE_INPUT" = "Y" ]; then
            REMOVE_EVIDENCE=true
            echo -e "${YELLOW}✓ Evidence files will be removed${NC}"
        fi
    fi
    
    echo ""
    echo -e "${RED}${BOLD}⚠ WARNING: This action cannot be undone!${NC}"
    printf "Proceed with uninstallation? [y/N]: "
    read -r CONFIRM
    if [ "$CONFIRM" != "y" ] && [ "$CONFIRM" != "Y" ]; then
        echo "Uninstallation cancelled."
        exit 0
    fi
    echo ""
}

stop_services() {
    echo -e "${BLUE}Stopping services...${NC}"
    
    # Stop systemd service
    if [ "$SYSTEMD_INSTALLED" = true ]; then
        if systemctl is-active "$SERVICE_NAME" >/dev/null 2>&1; then
            echo -e "${YELLOW}Stopping systemd service...${NC}"
            systemctl stop "$SERVICE_NAME" 2>/dev/null || true
        fi
    fi
    
    # Stop init.d services
    if [ "$INITD_INSTALLED" = true ]; then
        for service in $SERVICE_NAME $LEGACY_SERVICE; do
            if [ -f "/etc/init.d/$service" ]; then
                echo -e "${YELLOW}Stopping init.d service: $service${NC}"
                service "$service" stop 2>/dev/null || true
                /etc/init.d/"$service" stop 2>/dev/null || true
            fi
        done
    fi
    
    echo -e "${GREEN}✓ Services stopped${NC}"
    echo ""
}

remove_services() {
    echo -e "${BLUE}Removing service configurations...${NC}"
    
    # Remove systemd service
    if [ "$SYSTEMD_INSTALLED" = true ]; then
        echo -e "${YELLOW}Disabling systemd service...${NC}"
        systemctl disable "$SERVICE_NAME" 2>/dev/null || true
        
        if [ -f "/etc/systemd/system/$SERVICE_NAME.service" ]; then
            rm -f "/etc/systemd/system/$SERVICE_NAME.service"
            echo -e "${GREEN}✓ Removed systemd service file${NC}"
        fi
        
        systemctl daemon-reload 2>/dev/null || true
    fi
    
    # Remove init.d services
    if [ "$INITD_INSTALLED" = true ]; then
        for service in $SERVICE_NAME $LEGACY_SERVICE; do
            if [ -f "/etc/init.d/$service" ]; then
                echo -e "${YELLOW}Removing init.d service: $service${NC}"
                
                # Disable service
                if command -v update-rc.d >/dev/null 2>&1; then
                    update-rc.d "$service" remove 2>/dev/null || true
                elif command -v chkconfig >/dev/null 2>&1; then
                    chkconfig "$service" off 2>/dev/null || true
                fi
                
                # Remove service file
                rm -f "/etc/init.d/$service"
                echo -e "${GREEN}✓ Removed init.d service: $service${NC}"
            fi
        done
    fi
    
    echo ""
}

remove_cron_jobs() {
    echo -e "${BLUE}Removing cron jobs...${NC}"
    
    # Remove cron files
    for cron_file in ispprotect_bandaemon ispconfig3_bandaemon ispprotect_daemon; do
        if [ -f "/etc/cron.d/$cron_file" ]; then
            rm -f "/etc/cron.d/$cron_file"
            echo -e "${GREEN}✓ Removed cron job: $cron_file${NC}"
        fi
    done
    
    echo ""
}

remove_logrotate() {
    echo -e "${BLUE}Removing logrotate configuration...${NC}"
    
    # Remove logrotate files
    for logrotate_file in ispprotect_bandaemon ispconfig3_bandaemon; do
        if [ -f "/etc/logrotate.d/$logrotate_file" ]; then
            rm -f "/etc/logrotate.d/$logrotate_file"
            echo -e "${GREEN}✓ Removed logrotate config: $logrotate_file${NC}"
        fi
    done
    
    echo ""
}

remove_postlog_scripts() {
    echo -e "${BLUE}Removing postlog scripts...${NC}"
    
    # Remove from PHP directories
    for php_dir in "/usr/share/php" "/usr/share/php5" "/usr/share/php7.0" "/usr/share/php7.4" "/usr/share/php8.0" "/usr/share/php8.1"; do
        if [ -f "$php_dir/ispp_bd_postlog.php" ]; then
            rm -f "$php_dir/ispp_bd_postlog.php"
            echo -e "${GREEN}✓ Removed postlog script from $php_dir${NC}"
        fi
    done
    
    # Remove legacy location
    if [ -f "/opt/ispp_bd_postlog.php" ]; then
        rm -f "/opt/ispp_bd_postlog.php"
        echo -e "${GREEN}✓ Removed legacy postlog script${NC}"
    fi
    
    echo ""
}

remove_system_symlink() {
    echo -e "${BLUE}Removing system symlink...${NC}"
    
    # Remove symlink from /usr/local/bin
    SYMLINK_PATH="/usr/local/bin/ispp_bandaemon"
    
    if [ -L "$SYMLINK_PATH" ]; then
        rm -f "$SYMLINK_PATH"
        echo -e "${GREEN}✓ Removed symlink: $SYMLINK_PATH${NC}"
    elif [ -f "$SYMLINK_PATH" ]; then
        echo -e "${YELLOW}⚠ File exists at $SYMLINK_PATH (not a symlink)${NC}"
        printf "Remove it? [y/N]: "
        read -r REMOVE_FILE
        if [ "$REMOVE_FILE" = "y" ] || [ "$REMOVE_FILE" = "Y" ]; then
            rm -f "$SYMLINK_PATH"
            echo -e "${GREEN}✓ Removed file: $SYMLINK_PATH${NC}"
        fi
    else
        echo -e "${YELLOW}⚠ No symlink found at $SYMLINK_PATH${NC}"
    fi
    
    echo ""
}

remove_installation_files() {
    echo -e "${BLUE}Removing installation files...${NC}"
    
    # Remove main installation
    if [ "$MAIN_INSTALLED" = true ]; then
        # Handle configuration and evidence before removing directory
        if [ "$REMOVE_CONFIG" = false ] && [ -f "$INSTALL_DIR/config.inc.php" ]; then
            echo -e "${YELLOW}Backing up configuration to /tmp/ispprotect_config_backup.php${NC}"
            cp "$INSTALL_DIR/config.inc.php" "/tmp/ispprotect_config_backup.php" 2>/dev/null || true
        fi
        
        if [ "$REMOVE_CONFIG" = false ] && [ -f "$INSTALL_DIR/license.key" ]; then
            echo -e "${YELLOW}Backing up license to /tmp/ispprotect_license_backup.key${NC}"
            cp "$INSTALL_DIR/license.key" "/tmp/ispprotect_license_backup.key" 2>/dev/null || true
        fi
        
        if [ "$REMOVE_EVIDENCE" = false ] && [ -d "$INSTALL_DIR/evidence" ]; then
            echo -e "${YELLOW}Backing up evidence to /tmp/ispprotect_evidence_backup/${NC}"
            cp -r "$INSTALL_DIR/evidence" "/tmp/ispprotect_evidence_backup" 2>/dev/null || true
        fi
        
        # Remove SourceGuardian loader directory if it exists
        if [ -d "${INSTALL_DIR}/loader" ]; then
            rm -rf "${INSTALL_DIR}/loader"
            echo -e "${GREEN}✓ Removed SourceGuardian loader directory${NC}"
        fi
        
        # Remove directory
        rm -rf "$INSTALL_DIR"
        echo -e "${GREEN}✓ Removed main installation directory${NC}"
    fi
    
    # Remove legacy installation
    if [ "$LEGACY_INSTALLED" = true ]; then
        rm -rf "$LEGACY_DIR"
        echo -e "${GREEN}✓ Removed legacy installation directory${NC}"
    fi
    
    echo ""
}

remove_runtime_files() {
    echo -e "${BLUE}Removing runtime files...${NC}"
    
    # Remove PID files
    for pid_file in "/run/ispprotect_bandaemon.pid" "/var/run/ispconfig3_bandaemon.pid"; do
        if [ -f "$pid_file" ]; then
            rm -f "$pid_file"
            echo -e "${GREEN}✓ Removed PID file: $pid_file${NC}"
        fi
    done
    
    # Remove log files if requested
    if [ "$REMOVE_LOGS" = true ]; then
        for log_file in "/var/log/ispprotect_bandaemon.log" "/var/log/ispconfig3_bandaemon.log"; do
            if [ -f "$log_file" ]; then
                rm -f "$log_file"
                echo -e "${GREEN}✓ Removed log file: $log_file${NC}"
            fi
        done
    fi
    
    echo ""
}

remove_database() {
    if [ "$REMOVE_DATABASE" = true ]; then
        echo -e "${BLUE}Removing database...${NC}"
        
        if command -v mysql >/dev/null 2>&1; then
            # Try different authentication methods
            if [ -f "/etc/mysql/debian.cnf" ]; then
                echo -e "${YELLOW}Attempting database removal with debian.cnf...${NC}"
                mysql --defaults-file=/etc/mysql/debian.cnf -e "DROP DATABASE IF EXISTS ispprotect_bandaemon" 2>/dev/null
                mysql --defaults-file=/etc/mysql/debian.cnf -e "DROP USER IF EXISTS 'ispp_bandaemon'@'localhost'" 2>/dev/null
            else
                echo -e "${YELLOW}Please enter MySQL root password to remove database:${NC}"
                mysql -u root -p -e "DROP DATABASE IF EXISTS ispprotect_bandaemon"
                mysql -u root -p -e "DROP USER IF EXISTS 'ispp_bandaemon'@'localhost'"
            fi
            
            if [ $? -eq 0 ]; then
                echo -e "${GREEN}✓ Database removed successfully${NC}"
            else
                echo -e "${YELLOW}⚠ Database removal failed or skipped${NC}"
            fi
        else
            echo -e "${YELLOW}⚠ MySQL client not found, skipping database removal${NC}"
        fi
        
        echo ""
    fi
}

show_summary() {
    echo -e "${GREEN}${BOLD}Uninstallation completed!${NC}"
    echo ""
    
    if [ "$REMOVE_CONFIG" = false ]; then
        echo -e "${BLUE}Configuration backups (if any):${NC}"
        [ -f "/tmp/ispprotect_config_backup.php" ] && echo "• /tmp/ispprotect_config_backup.php"
        [ -f "/tmp/ispprotect_license_backup.key" ] && echo "• /tmp/ispprotect_license_backup.key"
        [ -d "/tmp/ispprotect_evidence_backup" ] && echo "• /tmp/ispprotect_evidence_backup/"
        echo ""
    fi
    
    if [ "$REMOVE_LOGS" = false ]; then
        echo -e "${BLUE}Remaining log files:${NC}"
        [ -f "/var/log/ispprotect_bandaemon.log" ] && echo "• /var/log/ispprotect_bandaemon.log"
        [ -f "/var/log/ispconfig3_bandaemon.log" ] && echo "• /var/log/ispconfig3_bandaemon.log"
        echo ""
    fi
    
    echo -e "${YELLOW}ISPProtect BanDaemon has been removed from your system.${NC}"
    echo -e "${YELLOW}Thank you for using ISPProtect BanDaemon!${NC}"
    echo ""
}

# Main uninstallation flow
main() {
    print_banner
    check_root
    detect_installation
    ask_user_preferences
    stop_services
    remove_services
    remove_cron_jobs
    remove_logrotate
    remove_postlog_scripts
    remove_system_symlink
    remove_installation_files
    remove_runtime_files
    remove_database
    show_summary
}

# Run main function
main "$@" 